package org.kmeans.bisec;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;

import org.kmeans.distances.DistanceStrategy;
/**
 * Coded by Luciano Giuseppe and Gino Farisano
 * @author Giuseppe Luciano
 *
 */
public class Bisec {
	
	public ArrayList<Document> clusters = null;
	private DistanceStrategy distance;
	private String datafile;
	
	public Bisec(String datafile, DistanceStrategy distance) {
		this.distance=distance;
		this.datafile = datafile;
	}

	/**
	 * 
	 * @param mainCenter: use -1 at first step
	 * @param prob
	 * @return
	 * @throws IOException
	 */
	public ArrayList<Document> getRandomCenters(int mainCenter, int prob) throws IOException {
				
		String row;
		int centrs = 0;
		Random rnd = new Random();
		ArrayList<Document> rndClusters = new ArrayList<Document>();
		ArrayList<Document> buffer = new ArrayList<Document>();
		
		int vicinCluster, counter;
		double minDist;
		Document tDoc;
		
		//get two random centroids
		int t;
		BufferedReader inData = new BufferedReader(new FileReader(this.datafile));
		while ( ((row = inData.readLine()) != null) && (centrs != 2) ) {
			
			//foreach doc
			tDoc = new Document(row, 1);
			
			vicinCluster = -1;
			counter = 0;
			minDist = Double.MAX_VALUE;
			
			//not the first kmeans
			if(mainCenter != -1) {
				for(Document cluster : this.clusters) {
					double tDist = this.distance.distance(tDoc, cluster);
					if(tDist < minDist) {
						vicinCluster = counter;
						minDist = tDist;
					}
					counter++;
				}
				
				//if not in mainCenter continue
				if(vicinCluster != mainCenter)
					continue;
			}
			
			//take a random doc
			t = rnd.nextInt(prob);
			if(t == rnd.nextInt(prob)) {
				centrs++; //count the centroid taken
				rndClusters.add(new Document(row, 1));
			} else if(rndClusters.size() < 2) {
				//use a buffer
				buffer.add(new Document(row, 1));
			}
		}
		
		inData.close();
		
		//not all centroids taken
		for(int i = rndClusters.size(); i < 2; i++)
			rndClusters.add(buffer.remove(0));
		
		return rndClusters;
	}
	
	/**
	 * 
	 * @param mainCntrs: -1 sto al primo step
	 * @param rndCentrs
	 * @throws IOException
	 */
	public ArrayList<Document> kmeans(int mainCntrs, ArrayList<Document> rndCentrs) throws IOException {
		
		//prepare two random centroids
		ArrayList<Document> newRndCenters = new ArrayList<Document>();
		newRndCenters.add(new Document());
		newRndCenters.add(new Document());
		
		String row;
		int nearCluster, counter;
		double minDist;
		Document tDoc;
		
		BufferedReader inData = new BufferedReader(new FileReader(this.datafile));	
		while ( (row = inData.readLine()) != null ) {
			
			//foreach document
			tDoc = new Document(row, 1);
			
			nearCluster = -1;
			counter = 0;
			minDist = Double.MAX_VALUE;
			
			//not the first kmeans
			if(mainCntrs != -1) {
				for(Document cluster : this.clusters) {
					double tDist = this.distance.distance(tDoc, cluster);
					if(tDist < minDist) {
						nearCluster = counter;
						minDist = tDist;
					}
					counter++;
				}
			}
			
			//re-assign docs in mainCntrs.
			if (mainCntrs == -1 || nearCluster == mainCntrs ) {
				
				nearCluster = -1; counter = 0;
				minDist = Double.MAX_VALUE;
				
				for(Document cluster : rndCentrs) {
					double tDist = this.distance.distance(tDoc, cluster);
					if(tDist < minDist) {
						nearCluster = counter;
						minDist = tDist;
					}
					counter++;
				}
				
				//prepare the new centroid
				newRndCenters.get(nearCluster).mergeDocument(tDoc);
			}
			
		} // end while
		inData.close();
		
		newRndCenters.get(0).normalize();
		newRndCenters.get(1).normalize();
		
		return newRndCenters;
	}

	/**
	 * Return the index of biggest cluster
	 * @return cluster index, or -1 at first step
	 */
	public int maxCluster() {
		if(clusters.size() == 0) return -1; //sto al primo step
		
		int max=clusters.get(0).getSize();
		int indexMax=0;
		int temp;
		for (int i = 1; i < clusters.size(); i++) {
			temp=clusters.get(i).getSize();
			if(temp>max){
				max=temp;
				indexMax=i;
			}
		}
		return indexMax;
	}
	
	
	
}
