package org.kmeans.bisec;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.io.Writer;
import java.util.ArrayList;

import org.kmeans.distances.CosiceDistance;
import org.kmeans.distances.DistanceStrategy;
import org.kmeans.distances.EuclideanDistance;

public class Main {

	public static void main(String[] args) throws IOException {

		System.out.println("Coded by Luciano Giuseppe And Gino Farisano (University of Salerno) \n");
		
		int clusters = 3; //Integer.parseInt(args[0]);
		int iterations = 4;//Integer.parseInt(args[1]); ecc...
		String fileName = "features.txt";
		int numbersOfDocuments = 13;

		DistanceStrategy strategy = null;
		int choice = 2;//Integer.parseInt(args[5]);

		if (choice == 1)
			strategy = new EuclideanDistance();
		else
			strategy = new CosiceDistance();

		Bisec bisec = new Bisec(fileName, strategy);


		//la probabilita' al primo ciclo
		int prob = numbersOfDocuments / 2;

		bisec.clusters = new ArrayList<Document>();

		// other bisec
		long startTime = System.currentTimeMillis();
		System.out.println("BisecKMeans on "+clusters+" cluster\n");
		int maxCentroid = -1; //at the start
		ArrayList<Document> subCentroidi = null;
		for (int i = 2; i <= clusters; i++) {
			System.out.println("Clusters: "+i);
			maxCentroid = bisec.maxCluster();

			// compute the prob.
			if(maxCentroid != -1) {
				prob = bisec.clusters.get(maxCentroid).getSize() / 2;
			}
			System.out.println("Gen centroids");
			subCentroidi = bisec.getRandomCenters(maxCentroid, prob);

			// make kmeans on two centroids
			for (int j = 0; j < iterations; j++) {
				System.out.println("Iteration: "+j);
				subCentroidi = bisec.kmeans(maxCentroid, subCentroidi);
			}

			if(maxCentroid != -1) {
				// remove the biggest cluster
				bisec.clusters.remove(maxCentroid);
			}
			// add the new two cluster
			bisec.clusters.addAll(subCentroidi);
			
			System.out.println();
		}
		long endTime = System.currentTimeMillis();
        System.out.println("It took " + (endTime - startTime) + " milliseconds");
		sortDocumentByCentroids(fileName, bisec.clusters, strategy);
		System.out.println("Finish");

	}

	/**
	 * Coded by Gino Farisano
	 */
	public static void sortDocumentByCentroids(String dataFile,
			ArrayList<Document> centroids, DistanceStrategy strategy)
			throws FileNotFoundException, UnsupportedEncodingException {
		Writer writerCentroids = new PrintWriter("centroids", "UTF-8");
		try {
			int k = centroids.size();
			for (Document item : centroids) {
				writerCentroids.append(item.getWords().toString()
						+ "Numbers of documents: " + item.getNumOfDocs());
				writerCentroids.append("\n\n");
			}
			writerCentroids.close();

			ArrayList<Writer> writerCentroidsFile = new ArrayList<Writer>();
			for (int i = 0; i < k; i++) {
				writerCentroidsFile.add(new PrintWriter("centroids" + i
						+ ".txt", "UTF-8"));
			}

			BufferedReader inData = new BufferedReader(new FileReader(dataFile));
			String rowVector;
			while ((rowVector = inData.readLine()) != null) {
				Document documentvector = new Document(rowVector, 2);
				double dist = Double.POSITIVE_INFINITY;

				int indexClusterCloser = 0;
				for (int i = 0; i < k; i++) {
					double temp = strategy.distance(
							(Document) centroids.get(i), documentvector);
					if (temp < dist) {
						indexClusterCloser = i;
						dist = temp;
					}
				}
				((Writer) writerCentroidsFile.get(indexClusterCloser))
						.append(documentvector.getName());
				((Writer) writerCentroidsFile.get(indexClusterCloser))
						.append("\n");
			}
			for (int i = 0; i < k; i++) {
				((Writer) writerCentroidsFile.get(i)).close();
			}
			inData.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
